@@ -22,7 +22,7 @@ use SUPER;
 
 # ABSTRACT: ECMAScript-262, Edition 5, lexical program grammar written in Marpa BNF
 
-our $VERSION = '0.015'; # VERSION
+our $VERSION = '0.017'; # VERSION
 
 our $WhiteSpace        = qr/(?:[\p{MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::CharacterClasses::IsWhiteSpace}])/;
 our $LineTerminator    = qr/(?:[\p{MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::CharacterClasses::IsLineTerminator}])/;
@@ -114,6 +114,15 @@ sub make_semantics_package {
 }
 
 
+sub spacesAny {
+    my $self = shift;
+    if (@_) {
+	$self->{_spacesAny} = shift;
+    }
+    return $self->{_spacesAny};
+}
+
+
 sub parse {
     my ($self, $source, $impl) = @_;
     #
@@ -143,6 +152,15 @@ sub _eventCallback {
   my %lastLexeme = ();
   my $lastLexemeDoneb = 0;
 
+  #
+  # Cache of some call results
+  #
+  $self->{_isIdentifierStart} = {};
+  $self->{_isDecimalDigit} = {};
+  $self->{_preSLength} = {};
+  $self->{_isEnd} = {};
+  $self->{_postLineTerminatorLength} = {};
+
   foreach (@{$impl->events()}) {
     my ($name) = @{$_};
     #
@@ -156,15 +174,15 @@ sub _eventCallback {
 	#
 	# Program$ will happen rarely, so even it does cost it is ok to do so
 	#
-	$self->{programCompleted} = $self->_isEnd($source, $pos, $impl);
+	$self->{programCompleted} = ($self->{_isEnd}->{$pos} //= $self->_isEnd($source, $pos, $impl));
     }
     elsif ($name eq 'NumericLiteral$') {
       #
       # The source character immediately following a NumericLiteral must not be
       # an IdentifierStart or DecimalDigit
       #
-      if ($self->_isIdentifierStart($source, $pos, $impl) ||
-          $self->_isDecimalDigit($source, $pos, $impl)) {
+      if (($self->{_isIdentifierStart}->{$pos} //= $self->_isIdentifierStart($source, $pos, $impl)) ||
+          ($self->{_isDecimalDigit}->{$pos} //= $self->_isDecimalDigit($source, $pos, $impl))) {
         my ($start, $end) = $impl->last_completed_range('NumericLiteral');
         my $lastNumericLiteral = $impl->range_to_string($start, $end);
         SyntaxError(error => "NumericLiteral $lastNumericLiteral must not be immediately followed by an IdentifierStart or DecimalDigit");
@@ -196,7 +214,7 @@ sub _eventCallback {
 	    $self->getLastLexeme(\%lastLexeme, $impl);
 	    $lastLexemeDoneb = 1;
 	}
-	my $Slength = $self->_preSLength($source, $rc, $impl);
+	my $Slength = ($self->{_preSLength}->{$rc} //= $self->_preSLength($source, $rc, $impl));
 	$self->_insertInvisibleSemiColon($impl, $rc, $Slength);
 	$rc += $Slength;
     }
@@ -209,7 +227,7 @@ sub _eventCallback {
 	    $lastLexemeDoneb = 1;
 	}
       my $postLineTerminatorPos = $lastLexeme{start} + $lastLexeme{length};
-      my $postLineTerminatorLength = $self->_postLineTerminatorLength($source, $postLineTerminatorPos, $impl);
+      my $postLineTerminatorLength = ($self->{_postLineTerminatorLength}->{$postLineTerminatorPos} //= $self->_postLineTerminatorLength($source, $postLineTerminatorPos, $impl));
       if ($postLineTerminatorLength > 0) {
 	  $impl->lexeme_read('SEMICOLON', $postLineTerminatorPos, $postLineTerminatorLength, ';');
       }
@@ -223,7 +241,7 @@ sub _eventCallback {
     # ^^DIV (because of REGULAREXPRESSIONLITERAL that can eat it)
     # -----------------------------------------------------------
     elsif ($name eq '^^DIV') {
-	my $realpos = $rc + $self->_preSLength($source, $rc, $impl);
+	my $realpos = $rc + ($self->{_preSLength}->{$rc} //= $self->_preSLength($source, $rc, $impl));
       if (index($source, '/',  $realpos) == $realpos &&
           index($source, '/=', $realpos) != $realpos &&
           index($source, '//', $realpos) != $realpos &&
@@ -236,7 +254,7 @@ sub _eventCallback {
     # ^^DIVASSIGN  (because of REGULAREXPRESSIONLITERAL that can eat it)
     # ------------------------------------------------------------------
     elsif ($name eq '^^DIVASSIGN') {
-	my $realpos = $rc + $self->_preSLength($source, $rc, $impl);
+	my $realpos = $rc + ($self->{_preSLength}->{$rc} //= $self->_preSLength($source, $rc, $impl));
       if (index($source, '/=', $realpos) == $realpos &&
           index($source, '//', $realpos) != $realpos &&
           index($source, '/*', $realpos) != $realpos) {
@@ -246,6 +264,15 @@ sub _eventCallback {
     }
   }
 
+  #
+  # Remove cache
+  #
+  delete($self->{_isIdentifierStart});
+  delete($self->{_isDecimalDigit});
+  delete($self->{_preSLength});
+  delete($self->{_isEnd});
+  delete($self->{_postLineTerminatorLength});
+
   #if ($rc != $pos) {
   #  $log->tracef('[_eventCallback] Resuming at position %d (was %d when called)', $rc, $pos);
   #}
@@ -371,22 +398,10 @@ sub _isDecimalDigit {
 sub _isEnd {
     # my ($self, $source, $pos, $impl) = @_;
 
-  my $rc = 0;
-
-  my $prevpos = pos($_[1]);
-  pos($_[1]) = $_[2];
-
-  if ($_[1] =~ $isEnd) {
-    $rc = 1;
-  }
-
-  #if ($rc) {
-  #  $log->tracef('[_isEnd] Only spaces up to the end');
-  #}
-
-  pos($_[1]) = $prevpos;
-
-  return $rc;
+    my $grammar     = $_[0]->spacesAny->{grammar};
+    my $impl        = $_[0]->spacesAny->{impl};
+    $grammar->parse($_[1], $impl, $_[2]);
+    return $grammar->endReached;
 }
 
 sub _insertSemiColon {
@@ -477,7 +492,7 @@ MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Program - ECMAScr
 
 =head1 VERSION
 
-version 0.015
+version 0.017
 
 =head1 SYNOPSIS
 
@@ -505,6 +520,10 @@ Returns the grammar. This will be injected in the Program's grammar.
 
 Class method that returns Program default recce semantics_package. These semantics are adding ruleId to all values, and execute eventually StringLiteral lexical grammar.
 
+=head2 spacesAny($self, $spacesAny)
+
+Getter/Setter of a SpacesAny grammar implementation, used internally by the Program grammar.
+
 =head2 parse($self, $source, $impl)
 
 Parse the source given as $source using implementation $impl.
